%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function  y = DrawDisc( N,p )
%y = DrawDisc( N,p )
%N is an integer.
%p is an M x 1 vector of probabilities (sum to 1).
%y is an N x 1 vector of random variables drawn from 1,...,M with
%probabilities given by p. 

assert(abs(sum(p)-1) < 1e-10)
assert(all(p>=0));
assert(all(p<=1));


M = numel(p);
pcs = cumsum(p(:));
Yinds = (1:M)';
if nnz(p>0) == 1
    y = find(p)*ones(N,1);
else
    y = interp1([0;pcs(p>0)],[1;Yinds(p>0)],rand(N,1),'next');
end


end

