%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [PC, XC,Ind] = MergeMarkovChains(PA,XA,PB,XB)
% [PC, XC] = MergeMarkovChains2(PA,XA,PB,XB)
% Merge two Markov chains
% All X variables are states vectors (optionally with more than one column
% corresponding to different variables)
% P variables are transition matrix with dimensions that conform to the
% rows of X
%
% Ind -- Array of indices that index XC such that
% [XA(i,:) XB(j,:)] == XC(Ind(i,j),:)


[mA, nA] = size(XA);
[mB, nB] = size(XB);


assert(all(size(PA) == [mA, mA]))
assert(all(size(PB) == [mB, mB]))

XC = [kron(XA,ones(mB,1)) kron(ones(mA,1),XB)];
PC = kron(PA,PB);

assert(all(size(XC) == [mA*mB, nA+nB]))

Ind = zeros(mB,mA);
Ind(:) = 1:mA*mB;
Ind = Ind';

