%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

close all; 
clear all;
clc;

 
for ESTCASE = 1:3
%-- estimation 1 --  No skill loss or gain
%-- estimation 2 --  Skill loss or gain, but not time-varying degree (no x)
%-- estimation 3 --  Baseline -- Skill loss or gain abd time-varying degree (x)
setup;


[ mod_mn1, mod_mn3, mod_mn5, ...
    mod_P101, mod_P103, mod_P105,...
    mod_P501, mod_P503, mod_P505,...
    mod_P901, mod_P903, mod_P905,...
    mod_sd1, mod_sd3, mod_sd5,...
    x, MassTrimmed, FullYearUnemp,...
    mod_skew1, mod_skew3, mod_skew5 ] =  simulate(pvec,lambda_s,lambda_l,zeta,t, nsim, omega, LS, X);


skew_dat_5 = ((P905-P505)-(P505-P105))./(P905-P105);
skew_mod_5 = ((mod_P905-mod_P505)'-(mod_P505-mod_P105)')./(mod_P905-mod_P105)';

skew_dat_1 = ((P901-P501)-(P501-P101))./(P901-P101);
skew_mod_1 = ((mod_P901-mod_P501)'-(mod_P501-mod_P101)')./(mod_P901-mod_P101)';

if ESTCASE == 3
    figure('Position',[0 0 600 600])
    m = {'mn','P10','P50','P90'};
    m_desc = {'mean','P10','P50','P90'};
    d = {'1','3','5'};
    for im = 1:4
        for id = 1:3
            tm = 1978:2011-str2num(d{id});
            subplot(4,3,id + 3*(im-1))
            eval(['plot(tm,100*mod_' m{im} d{id} ',''k'')'])
            hold on
            eval(['plot(tm,' m{im} d{id} ')'])
            hold off
            children = get(gca,'Children');
            set(children(1),'Color',[ 0.65 0.65 0.65])
            xlim([1975 2015])
            if id == 1
                yl = get(gca,'yLim');
                h1 = text(1960,yl(1) + 0.4*diff(yl),m_desc{im});
                set(h1,'rotation',90);
            end

            if im == 1
                eval(['title(''' d{id} '-year income change'')'])
            end
        end
    end






    % Davis von Wachter
    [Recess,Expan] = DavisVonWachter(untransform(pvec),x,t);
    disp('Davis von Wachter')
    disp(['Recess = ' num2str(Recess)])
    disp(['Expan  = ' num2str(Expan)])

    % Report correlation of x and initial claims

    ql = load('quarterly_labor.mat');
    IC = ql.initialclaims(ql.t >=1977 & ql.t < 2012);
    cc = corrcoef(IC,x);
    disp(['Correlation of initial claims and x = ' num2str(cc(1,2))])


    % save the results
    save quarterly_labor_out.mat x lambda_s lambda_l zeta;
end


save(['skew_plots_' num2str(ESTCASE)  '.mat'], 'skew_dat_1','skew_dat_5','skew_mod_1','skew_mod_5')

end

plotx
PlotKelleyWithAlternatives
