%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

clear all
close all
clc

startDate = 1977.0;
endDate = 2011.75;

[lambda_s, lambda_l, zeta, ~, ~, t] = new_lambda_zeta(startDate,endDate);


load('quarterly_labor_out.mat','x');


X = [invlogistic(zeta) invlogistic(lambda_s)  x];

m = mean(X);

Xdm = bsxfun(@minus,X,m);

XL = Xdm(1:end-1,:);
Xdm = Xdm(2:end,:);




A = zeros(3,3);


for i = 1:3
    XLi = XL(:,i);
    A(i,i) = (XLi' * XLi) \ XLi' * Xdm(:,i);
end


eps = Xdm - XL * A';
covmat = cov(eps);

B = chol(covmat);
B = B';
assert(all(all(abs(covmat - B * B') < 1e-12)))


LambdaEllFactor = mean(lambda_l) / mean(lambda_s);


%system has form
%X_t - m  =  A * (X_{t-1} - m) + B * eps
% where eps have unit st dev 



save VAR_results.mat A B m LambdaEllFactor;

%-------------------------
% Test with simulation
%------------------------
T = 10000;
XX = zeros(3,T);
eps =  randn(3,T);
for t = 2:T
    XX(:,t) = A*XX(:,t-1) + B*eps(:,t);
end
XX = XX(:,T/10+1:end);
XX = bsxfun(@plus,XX',m);


    