%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function ob = objective( pvec_est, pvec_tot, lambda_s, lambda_l, zeta, t, nsim, omega, LS,X,...
        estInd,...
        mn1, mn3, mn5, ...
        P101, P103, P105,...
        P501, P503, P505,...
        P901, P903, P905 )
% computes objective for one parameter vector


pvec = pvec_tot;
pvec(estInd) = pvec_est;



rng('default')
rng(14234986)



[ mod_mn1, mod_mn3, mod_mn5, ...
    mod_P101, mod_P103, mod_P105,...
    mod_P501, mod_P503, mod_P505,...
    mod_P901, mod_P903, mod_P905,...
    ~,~,~,x] =  simulate(pvec,lambda_s,lambda_l,zeta,t, nsim, omega, LS, X);




%compute moments and their deviations
mod_mn1 = mod_mn1*100;
mod_mn3 = mod_mn3*100;
mod_mn5 = mod_mn5*100;
mod_P101 = mod_P101*100;
mod_P103 = mod_P103*100;
mod_P105 = mod_P105*100;
mod_P501 = mod_P501*100;
mod_P503 = mod_P503*100;
mod_P505 = mod_P505*100;
mod_P901 = mod_P901*100;
mod_P903 = mod_P903*100;
mod_P905 = mod_P905*100;




ob = 0;

%means
%ob = ob + sum(((mod_mn1 - mn1')/mean(mod_P901)).^2);
%ob = ob + sum(((mod_mn3 - mn3')/mean(mod_P903)).^2);
%ob = ob + sum(((mod_mn5 - mn5')/mean(mod_P905)).^2);

% % P10
% ob = ob + sum(((mod_P101 - P101')./mod_P101).^2);
% ob = ob + sum(((mod_P103 - P103')./mod_P103).^2);
% ob = ob + sum(((mod_P105 - P105')./mod_P105).^2);
% 
% P50
ob = ob + sum(((mod_P501 - P501')/mean(mod_P901)).^2);
ob = ob + sum(((mod_P503 - P503')/mean(mod_P903)).^2);
ob = ob + sum(((mod_P505 - P505')/mean(mod_P905)).^2);

% %P90
% ob = ob + sum(((mod_P901 - P901')./mod_P901).^2);
% ob = ob + sum(((mod_P903 - P903')./mod_P903).^2);
% ob = ob + sum(((mod_P905 - P905')./mod_P905).^2);


%left-tail
ob = ob + sum((((mod_P501-mod_P101)' - (P501-P101))./(P501-P101)).^2);
ob = ob + sum((((mod_P503-mod_P103)' - (P503-P103))./(P503-P103)).^2);
ob = ob + sum((((mod_P505-mod_P105)' - (P505-P105))./(P505-P105)).^2);

%right-tail
ob = ob + sum((((mod_P901-mod_P501)' - (P901-P501))./(P901-P501)).^2);
ob = ob + sum((((mod_P903-mod_P503)' - (P903-P503))./(P903-P503)).^2);
ob = ob + sum((((mod_P905-mod_P505)' - (P905-P505))./(P905-P505)).^2);


%average position of P90 and P10
ob = ob + (mean(mod_P901)/mean(P901) - 1)^2;
ob = ob + (mean(mod_P101)/mean(P101) - 1)^2;
ob = ob + (mean(mod_P903)/mean(P903) - 1)^2;
ob = ob + (mean(mod_P103)/mean(P103) - 1)^2;
ob = ob + (mean(mod_P905)/mean(P905) - 1)^2;
ob = ob + (mean(mod_P105)/mean(P105) - 1)^2;


% Davis von Wachter moments
[DvWRecess,DvWExpan] = DavisVonWachter( untransform(pvec),x,t);
ob = ob + ( (DvWRecess - (-0.2))/0.2)^2;
ob = ob + ( (DvWExpan  - (-0.1))/0.1)^2;


% 
% skew_dat3 = ((P903-P503)-(P503-P103))./(P903-P103);
% skew_mod3 = ((mod_P903-mod_P503)'-(mod_P503-mod_P103)')./(mod_P903-mod_P103)';
%  
% skew_dat5 = ((P905-P505)-(P505-P105))./(P905-P105);
% skew_mod5 = ((mod_P905-mod_P505)'-(mod_P505-mod_P105)')./(mod_P905-mod_P105)';
% 
% ob = ob + sum((skew_dat3 - skew_mod3).^2);
% ob = ob + sum((skew_dat5 - skew_mod5).^2);

end

