%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

addpath('Library')


load('TableA13.mat','mn1','mn3','mn5','P101','P103','P105','P501','P503','P505','P901','P903','P905');


% select start and end dates
startDate = 1977.0;
endDate = 2011.75;
[lambda_s, lambda_l, zeta, ParticipationRate, AvgWeeklyHours,U_short, U_med, U_long, t] = new_lambda_zeta(startDate,endDate);
T = length(t);

% create the matrix of factors for the mean income change
CenterX = [ [zeros(8,1); kron(mn1,ones(4,1))/400]  lambda_s];
CenterX = [ones(size(zeta)) MyPCA( CenterX )];

% create the matrix of factors for the tail risks
xX = MyPCA([U_short U_long AvgWeeklyHours ParticipationRate]);

%Store in structure
X.Center = CenterX;
X.x = xX;
clear CenterX xX;


omega = 1/160;
nsim = 100000;


% compute steady state labor market stocks
ls = mean(lambda_s);
ll = mean(lambda_l);
z = mean(zeta);
A = [1-z ls ll; z 0 0; 0 1-ls 1-ll];  % transition matrix between e, us, ul
opts.disp=0;
[LS,egval] = eigs(A,[],1,1+1e-10,opts);
assert(abs(egval-1)<1e-10);
LS = LS/sum(LS);








if ESTCASE == 1
    pvec = [  0;    % p2 -- prob(skill loss | EU)               1
          0;   % p3 -- prob(skill gain | E)                 2
          0.144313651962846; % sigmae -- std trans shock               3
          0.110577555397273;  % sigma1 -- std center shock             4
          0.1041;  % sigma2,3 -- std tail shocks            5
          -0.2508; % mean of job loss shock                 6
          0.3071;  % mean of skill gain shock               7
 % four coefs on avg inc growth 8:11.          
          0.011138123791639;
          0.001987582942857;
         -0.001397526252054;
         0;
% four coefs for x 12:15.
          0;
          0;
          0;
          0];
     
    % Which parameter indices do we optimize over?
    estInd = [3 4 8:10];
    
elseif ESTCASE == 2  
    pvec = [   0.334879306090049;    % p2 -- prob(skill loss | EU)               1
          0.038951403262053;   % p3 -- prob(skill gain | E)                 2
          0.145523205988315; % sigmae -- std trans shock               3
          0.093340417663110;  % sigma1 -- std center shock             4
          0.114591275401318;  % sigma2,3 -- std tail shocks            5
          -0.349359670787609; % mean of job loss shock                 6
          0.230319138671907;  % mean of skill gain shock               7
% four coefs for avg inc growth 8:11.          
          0.010607545568002;
          0.002987075386130;
         -0.001305463014850;
          0;
% four coefs for x 12:15.
          0;
          0;
          0;
          0];
     
     % Which parameter indices do we optimize over?
    estInd = 1:10;
    
    
elseif ESTCASE == 3  

  
  
        pvec = [   0.714516380391143;    % p2 -- prob(skill loss | EU)               1
                   0.018911617365324;    % p3 -- prob(skill gain | E)                2
                   0.154540058789747;    % sigmae -- std trans shock                 3
                   0.063777918079094;    % sigma1 -- std center shock                4
                   0.333728020205876;    % sigma2,3 -- std tail shocks               5
                  -0.166868386289571;    % mean of job loss shock                    6
                   0.393951183726230;    % mean of skill gain shock                  7
% four coefs for avg inc growth 8:11.          
   0.012326139306866;
   0.002544705758662;
  -0.001516592314434;
   0.019875636653074;
% four coefs for x 12:15.
    -0.185209852040809;
  -0.146903385829873;
  -0.038585197998301;
   0.008435071779274];

     % Which parameter indices do we optimize over?
    estInd = 8:15;
  


end

pvec = transform(pvec);

np = length(pvec);

