%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [ mn1, mn3, mn5, ...
    P101, P103, P105,...
    P501, P503, P505,...
    P901, P903, P905,...
    sd1, sd3, sd5,...
    x, MassTrimmed, FullYearUnemp,...
    skew1, skew3, skew5] =  simulate(pvecarg,lam_s,lam_l,zeta,t, nsim, omega,InitialStocks ,X)


Stocks = zeros(3,size(t,1)+1);
Stocks(:,1) = InitialStocks;
for it = 2:numel(t)+1
    A = GetTransMat(lam_s(it-1),lam_l(it-1),zeta(it-1));
    Stocks(:,it) = A * Stocks(:,it-1);
end


%unpack
pvec = untransform(pvecarg);

p2 = pvec(1);
p3 = pvec(2);


sigmae = pvec(3);
sigma1 = pvec(4);
sigma2 = pvec(5);
sigma3 = sigma2;

a1 = 0;
a2 = pvec(6);
a3 = pvec(7);


x = X.x*pvec(12:15);
pos = X.Center*pvec(8:10) - pvec(11)*x;


mu1 = a1;
mu2 = a2 - x;
mu3 = a3 - x;

p1 = 1 - p2*zeta.*Stocks(1,1:end-1)' - p3*Stocks(1,2:end)';
mubar = -log(p1.*exp(mu1+sigma1^2/2) + p2*zeta.*Stocks(1,1:end-1)'.*exp(mu2+sigma2^2/2) + p3*Stocks(1,2:end)'.*exp(mu3+sigma3^2/2));
mu1 = mu1 + mubar;
mu2 = mu2 + mubar;
mu3 = mu3 + mubar;


mu1 = mu1 + pos;
mu2 = mu2 + pos;
mu3 = mu3 + pos;


T = length(t);
theta = zeros(nsim,T+1);
dead = false(nsim,T+1);
e = zeros(nsim,T+1);
transclass = zeros(nsim,1);

e(:,1) =  DrawDisc( nsim, InitialStocks);



for it = 2:T+1
    %shock to employment
    A = GetTransMat(lam_s(it-1),lam_l(it-1),zeta(it-1));
    for i = 1:3
        I = e(:,it-1) == i;
        nI = nnz(I);
        if nI > 0
            e(I,it) = DrawDisc(nI,A(:,i));
        end
    end
    
    % classify transitions
    transclass(:) = 1;
    transclass(e(:,it-1) == 1  & e(:,it) > 1) = 2;  % job separators
    transclass(e(:,it) == 1 ) = 3;  % employed today
    % residual category 1: unemployed last period and this period.
    
    
    % --- draw shock 1, 2, or 3 ---
    % of those in class 2, fraction p2 get shock 2, 1-p2 get shock 1
    I = find(transclass == 2);
    n2 = length(I);
    transclass(randsample(I,round((1-p2)*n2))) = 1;
    n2 = n2 - round((1-p2)*n2);
    
    % of those in class 3, fraction p3 get shock 3, 1-p3 get shock 1
    I = find(transclass == 3);
    n3 = length(I);
    transclass(randsample(I,round((1-p3)*n3))) = 1;
    n3 = n3 - round((1-p3)*n3);
    
    n1 = nsim - n2 - n3;
    
    % generate the shocks
    s1 = transclass == 1;
    s2 = transclass == 2;
    s3 = transclass == 3;
    
    theta(s1,it) = theta(s1,it-1) + mu1(it-1) + sigma1*randn(n1,1);
    theta(s2,it) = theta(s2,it-1) + mu2(it-1) + sigma2*randn(n2,1);
    theta(s3,it) = theta(s3,it-1) + mu3(it-1) + sigma3*randn(n3,1);
    
    
    %death:
    dead(:,it) = rand(nsim,1) < omega;
    theta(dead(:,it),it) = 0;
end

% % compute unemployment duration
% Dur = ~e(:,1);
% NDur = zeros(5,T);
% for t = 1:T
%     Dur = (Dur+1) .* (~e(:,t+1));
%     NDur(:,t) = histcounts(Dur,[-0.5, 0.5, 1.5, 2.5, 4.5, 1e3]);
% end
% NDur = NDur/nsim;
% error('stop here')

theta = theta(:,2:end);
e = e(:,2:end);

%we only need data from 1978 to 2011
I = t>=1978 & t<2012;
theta = theta(:,I);
e = e(:,I);
dead = dead(:,I);
T1 = length(1978:2011);

%add transitory shock
var4sigmae = 4*sigmae^2;
income = (e==1).*exp(theta -var4sigmae/2 + sqrt(var4sigmae)*randn(nsim,4*T1));
MeanIncome = mean(income(:));
LowEarningsCutoff = 1300/50000 * MeanIncome;

%aggregate to annual data
y = zeros(nsim,T1);
MassTrimmed = zeros(1,T1);
FullYearUnemp = zeros(1,T1);
for it = 1:T1
    I = 4*(it-1)+(1:4);
    y(:,it) = log(sum(income(:,I),2));
    iszero = y(:,it) < log(LowEarningsCutoff);
    y(iszero,it) = NaN;
    y(any(dead(:,I),2),it) = NaN;
    MassTrimmed(it) = nnz(iszero)/nsim;
    FullYearUnemp(it) = nnz(all( e(:,I) ~= 1, 2)) / nsim;
end



%take differences across 1 year
d = 1;
dy1 = y(:,1+d:end) - y(:,1:end-d);
%note if either year contains a death, dy1 is NaN

%take differences across 3 years
d = 3;
dy3 = y(:,1+d:end) - y(:,1:end-d);
for it = 1:T1-d
    dy3(any(isnan(y(:,it:it+d)),2),it) = NaN;
end

%take differences across 5 years
d = 5;
dy5 = y(:,1+d:end) - y(:,1:end-d);
for it = 1:T1-d
    dy5(any(isnan(y(:,it:it+d)),2),it) = NaN;
end

% compute moments

%one-year differences
mn1 = zeros(1,T1-1);
sd1 = zeros(1,T1-1);
skew1 = zeros(1,T1-1);
P101 = zeros(1,T1-1);
P501 = zeros(1,T1-1);
P901 = zeros(1,T1-1);
for it = 1:T1-1
    I = ~isnan(dy1(:,it));
    mn1(it) = mean(dy1(I,it));
    sd1(it) = std(dy1(I,it));
    skew1(it) = skewness(dy1(I,it));
    P = prctile(dy1(I,it),[10 50 90]);
    P101(it) = P(1);
    P501(it) = P(2);
    P901(it) = P(3);
end


%three-year differences
mn3 = zeros(1,T1-3);
sd3 = zeros(1,T1-3);
skew3 = zeros(1,T1-3);
P103 = zeros(1,T1-3);
P503 = zeros(1,T1-3);
P903 = zeros(1,T1-3);
for it = 1:T1-3
    I = ~isnan(dy3(:,it));
    mn3(it) = mean(dy3(I,it));
    sd3(it) = std(dy3(I,it));
    skew3(it) = skewness(dy3(I,it));
    P = prctile(dy3(I,it),[10 50 90]);
    P103(it) = P(1);
    P503(it) = P(2);
    P903(it) = P(3);
end

%five-year differences
mn5 = zeros(1,T1-5);
sd5 = zeros(1,T1-5);
skew5 = zeros(1,T1-5);
P105 = zeros(1,T1-5);
P505 = zeros(1,T1-5);
P905 = zeros(1,T1-5);
for it = 1:T1-5
    I = ~isnan(dy5(:,it));
    mn5(it) = mean(dy5(I,it));
    sd5(it) = std(dy5(I,it));
    skew5(it) = skewness(dy5(I,it));
    P = prctile(dy5(I,it),[10 50 90]);
    P105(it) = P(1);
    P505(it) = P(2);
    P905(it) = P(3);
end

end