%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function s  = AMpolyint_step1( X, ord, classification, s0 )
% s = AMpolyint_step1( X, deg )
% Sets up a polynomial interpolation structure.
% In: X     n x d matrix of points on which we will fit the polynomial (ie
%           the grid).  rows are observation and columns are variables.
%
% ord       the order of the polynomial approximation. can be 1, 2, or 1.5
%           (for mixed)
%           classification 1 x d array.  It serves two purposes: first,
%           zero entries in classification result in variables that are
%           dropped.  This is useful when we want to simulate variables
%           and treat them as states but they are not used in the
%           interpolation because they are not actually states.  The reason
%           for this treatment is that they are states in some
%           specifications but not others and we don't want to write the
%           code treating them in different ways.  Second, if ord = 1.5,
%           then classification takes values 1 or 2 to indicate which
%           variables are treated which way.
%
% s0        a previous interpolation structure to be used for an initial
%           guess of coefficients.
%
% Out: s    a structure for later calls

s.d = nnz(classification);
s.ord = ord;
if exist('classification','var') && ~isempty(classification)
    s.clsmask = classification ~= 0;
    s.cls = classification;
else
    s.clsmask = true(1,s.d);
    s.cls = ord * ones(1,s.d);
end

s.B = AMpolyint_basis(X(:,s.clsmask),ord,s.cls(s.clsmask));

if exist('s0','var') 
    yy = AMpolyint_basis(X(:,s0.clsmask),s0.ord,s0.cls(s.clsmask)) * s0.coef;
    s = AMpolyint_step2(s,yy);
end
    

    

end

