%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [grid, eps, PCgrid] = EDS_get_grid(state,thin)
%Creates a grid from simulated data
% state is a n x T matrix of simulated state variables with each column
% corresponding to time period and each row corresponding to a state
% variable.
%
% Before creating the grid, the function drops a fraction of the time periods.
% thin is a number in [0,1] that says what fraction to keep.  E.g. thin =
% 0.1 says keep every tenth period.
%
% Outputs
% grid is 

global Params;

nState = size(state,1);

sim_T = size(state,2);

%EDS 1-thin to [thin]%
EDSSIMt = thin*sim_T;
I = false(1,1/thin);
I(1) = true;
I = repmat(I,1,EDSSIMt);
state = state(:,I)';
clear I;

%%
% EDS 2 - pricnipal components
statemean = mean(state);
ss = bsxfun(@minus,state,statemean);
statestd = std(state);
ss = ss*diag(1./statestd);  

[~,~,V] = svd(ss); % produces a diagonal matrix S, of the same 
                  % dimension as X and with nonnegative diagonal elements in
                  % decreasing order, and unitary matrices U and V so that X = U*S*V'
PC = ss*V;    
D = std(PC);
PC = PC * diag(1./D);
PC2ss = diag(D)*V';

tmp = corrcoef(PC) - eye(size(PC,2));

clear tmp;

%% EDS 3 - eliminate low probability points
disp('EDS -- computing densities')
EDSh = EDSSIMt^(-1/(nState+4));  %bandwidth
EDSdelta = 0.025;  % fraction to drop

dens = zeros(EDSSIMt,1);

for i = 1:EDSSIMt
    dens(i) = (1/(EDSSIMt*(2*pi)^(nState/2)*EDSh^nState))*sum(exp(-EDS_dist(PC,PC(i,:)))/(2*EDSh^2));
end
[~,I] = sort(dens);


%discard lowest %
ndisc = round(EDSSIMt*EDSdelta);
PC = PC(I(ndisc+1:end),:);

size(PC)



%% EDS 4 - create eps grid
[Pe, eps] = MEpsGrid(PC,Params.EDSM, 1, [0.3 10]);


M = length(Pe);
grid = Pe*PC2ss*diag(statestd) + repmat(statemean,M,1);
PCgrid = Pe;
