%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [EEE_mat, testGrid] = EulerErrorNL(S0,dist, polfunc, nerr)
%EULERERROR Simulates the economy and computes Euler equation errors at
%randomly generated state vectors

global Params;

Params.Tsim = 1000;
nxtest = 1e3;
testGrid = linspace(Params.xmin,Params.xmax,nxtest)';

EEE_mat = NaN(nxtest,Params.npp,nerr);

AggEps = randn(Params.nz,Params.Tsim+Params.burn,nerr);  
Sigma = Params.Sigma;

% parfor here   
parfor ierr = 1:nerr
    
    disp(['iteration number: ' num2str(ierr)])
    
    % Generate shocks for business cycle simulations
    
    AggEpsi = diag(Sigma)*AggEps(:,:,ierr);
    [state, finalDist] = SimulateDynamics(polfunc,S0,dist,AggEpsi,Params); 

    
    
    EEE  = EulerError_at_a_state( state(:,end-1:end), polfunc, finalDist, testGrid, Params);
    
    
    EEE_mat(:,:,ierr) = EEE;
end

end

function [ EEE ] = EulerError_at_a_state( SS, polfunc, dist, testGrid, Params)

S = SS(:,2)';
Slag = SS(:,1)';

agridUE = AMpolyint_step3(polfunc,S);
Kp = SimulateCoreNonStoch(polfunc,S,Slag,dist,Params);

savings = zeros(numel(testGrid),Params.npp);
constrained = false(size(savings));
for ip = 1:Params.npp
    ind = (ip-1)*Params.nkp;
    agrid = agridUE(ind+(1:Params.nkp));
    savings(:,ip) = interpagridkpgrid(agrid,Params.kpgrid,testGrid);
    constrained(:,ip) = testGrid <= agrid(1);   
end
CLHS = bsxfun(@minus,testGrid, savings);

CRHS = eulerresNL(polfunc,Kp,Params, S,savings);


EEE = CRHS./CLHS -1;
EEE(constrained) = 0;




end


