%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

%Run this afterwards in order to compute Euler equation Errors.


nerr = 100;


[EEE_mat, testGrid] = EulerErrorNL(S0, finalDist, polfunc, nerr);

nxtest = length(testGrid);


maxerr = max(abs(EEE_mat),[],3);
mnerr = mean(abs(EEE_mat),3);
save EulerErrorNLResults.mat maxerr mnerr EEE_mat testGrid;

%% Plots

fh = figure('Position',[1 1 600 800]);
plot_order = [1 3 5 2 4 6];
plot_titles = {'Impatient - employed', 'unemployed', 'long-term unemployed', ...
               'Patient - employed', 'unemployed', 'long-term unemployed'};
for i = 1:Params.npp
    subplot(ceil(Params.npp/2),2,plot_order(i),'FontSize',14);
    hold on;
    plot(testGrid,log10(mnerr(:,i)),'color','k','linewidth',1)
    plot(testGrid,log10(maxerr(:,i)),'color',[0,0,0]+0.5,'linewidth',1)
    hold off
    title(plot_titles{i});
    if i == 1
        lgh = legend('Mean','Max.');
        set(lgh,'FontSize',14)
    end
    if i == 3 || i == 6
        xlabel('Normalized cash on hand','FontSize',14)
    end
    if i < 4
        ylabel('Absolute error (log base 10)','FontSize',14)
    end
    box on;
    ylim([-5 -1.5])
end
saveas(fh,'EulerErrorNLFigure.fig')

