%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function HCheck( S0, D0, polfunc, forecastRule)
% Den Haan (2010) accuracy plot

global Params;

Timplied = 1000;


rng(72349)  
AggEps = diag(Params.Sigma(1:Params.nz))*randn(Timplied+200,Params.nz)';
state = SimulateDynamics(polfunc,S0,D0,AggEps,Params,200);
clear AggEps;



iK = nameInd('k',Params.aggnames);

impliedK = zeros(1,Timplied);
impliedK(1) = state(iK,1);
for t = 1:Timplied-1
    
    S = state(:,t)';
    S(iK) = impliedK(t);
    
    impliedK(t+1) = AMpolyint_step3(forecastRule,S);
end

K = state(iK,1:Timplied);

f = figure;
axes('FontSize',14)
hold on
plot(K,'-k')
plot(impliedK,'.k')
hold off
legh = legend('simulated', 'forecasting rule','Location','Best');
set(legh,'FontSize',14);
box on
saveas(f,'KS_accuracy.fig')

disp(['maximum abs error ' num2str(max(abs(log(K)-log(impliedK))))])
disp(['mean abs error ' num2str(mean(abs(log(K)-log(impliedK))))])



end

