

%----------------------------------------------------------------
% 0. Housekeeping (close all graphic windows)
%----------------------------------------------------------------

close all;

%----------------------------------------------------------------
% 1. Defining variables
%----------------------------------------------------------------

var z zetahat lambdahat k ushat ulhat us ul zeta lambda y c Rtilde w;
varexo ez eL1 eL2;

parameters beta delta alpha rhoz 
     sigmaz gamma
     laborAzeta laborAlambda 
     B11 B21 B22 
     labormlambda labormzeta
     lambdaLFactor
     tmp_usOnul;


%----------------------------------------------------------------
% 2. Calibration
%----------------------------------------------------------------

beta = 0.993;
alpha = 0.36;
delta = 0.02;
gamma = 2;
 
rhoz = 0.962;
sigmaz = 0.0081;


laborAzeta =  0.9363;
laborAlambda = 0.9240;


B11 =  0.0428 ;      
B21 = -0.0380 ; 
B22 =  0.0778 ;       
   


labormlambda =  0.7679;
labormzeta =  -3.0485;

lambdaLFactor = 0.5832;

tmp_usOnul = 1/(1+exp(-labormlambda));
tmp_usOnul = lambdaLFactor * tmp_usOnul / (1-tmp_usOnul);

    
%----------------------------------------------------------------
% 3. Model
%----------------------------------------------------------------

model; 
zetahat =  labormzeta + laborAzeta * (zetahat(-1) - labormzeta )  + B11 * eL1;
lambdahat = labormlambda +  laborAlambda * (lambdahat(-1) - labormlambda) + B21 * eL1 + B22 * eL2;
lambda = 1/(1+exp(-lambdahat));
zeta = 1/(1+exp(-zetahat));
ushat = -log(1/us -1);
ulhat = -log(1/ul -1);
us =  zeta * (1-us(-1)-ul(-1));
ul = (1-lambda)*us(-1) + (1-lambda*lambdaLFactor)*ul(-1);
y = exp(z)*k(-1)^alpha*(1-us-ul)^(1-alpha);
z = rhoz * z(-1) + ez;
c + k = y + (1-delta)*k(-1);
Rtilde = alpha*exp(z) * k(-1)^(alpha-1)*(1-us-ul)^(1-alpha) + 1 - delta;
w = (1-alpha) *exp(z) * k(-1)^alpha * (1-us-ul)^(-alpha);
c^(-gamma) = beta*c(+1)^(-gamma)*Rtilde(+1);
end;

%----------------------------------------------------------------
% 4. Computation
%----------------------------------------------------------------

initval;
lambdahat = labormlambda;
lambda = 1/(1+exp(-lambdahat));
zetahat = labormzeta;
zeta = 1/(1+exp(-zetahat));
ul = zeta/(tmp_usOnul*(1+zeta)+zeta);
us = tmp_usOnul * ul;
ulhat = -log(1/ul -1);
ushat = -log(1/us -1);
z = 0;
Rtilde = 1/(beta);
k = ((Rtilde - 1 + delta)/(alpha * exp(z))/((1-us-ul)^(1-alpha)))^(1/(alpha-1));
y = exp(z)*k^alpha*(1-us-ul)^(1-alpha);
c = y + (1-delta) * k  - k;
w = (1-alpha)*exp(z) * k^alpha * (1-us-ul)^(-alpha);
end;

shocks;
var ez = sigmaz^2;
var eL1 = 1;
var eL2 = 1;
end;

steady;
check;

stoch_simul(order = 1, periods=1000);



