%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

% Plots of full sample estimation
close all
clc
clear all


startdate = 1977.25;
enddate = 2011.5;


addpath(genpath('share'));

IsPresentation = false;


if IsPresentation
    FontSize = 14;
    LineWidth = 2;
else
    LineWidth = 1;
    FontSize = 12;
end

simstates = nan;
load(['data/EmpiricalSimulation_1977__betaHet_1_dox_1_doTails_1.mat'],'c', 'y','simstates','CYPcts')
c_base =  log(c'); %log(c') - hpfilter(log(c'),1600);
y_base = log(y'); %log(y') - hpfilter(log(y'),1600);
I_base =  log(y'-c'); %log(y'-c') - hpfilter(log(y'-c'),1600);



load(['data/EmpiricalSimulation_1977__RepAgent.mat'], 'c', 'y');
c_RA =  log(c'); %log(c') - hpfilter(log(c'),1600);
y_RA = log(y'); %log(y') - hpfilter(log(y'),1600);
I_RA = log(y'-c'); %log(y'-c') - hpfilter(log(y'-c'),1600);



load('data/consumption_data.mat','c','t')
c = log(c) - hpfilter(log(c),1600);
c_data = c(t>=startdate & t <= enddate);
t = t(t>=startdate & t <= enddate);


%% plots

colors = {repmat(0,1,3) , repmat(0,1,3),  repmat(0.5,1,3)};
styles = {'--','-','-'};
widths = {1,2,1};
markers = {'','','.'};
FontSize = 12;

fig = figure;
set(gcf,'Name','Cons')
p = plot(t, [c_data c_base-mean(c_base) c_RA-mean(c_RA)]);
for i = 1:3
    set(p(i),'color',colors{i});
    set(p(i),'LineStyle',styles{i});
    set(p(i),'LineWidth',widths{i});
    if ~strcmp(markers{i},'')
        set(p(i),'Marker',markers{i});
        set(p(i),'MarkerSize',20);
    end
end
legend('Data', 'Baseline', 'Complete Markets', 'Location','South');
set(gca,'FontSize',FontSize);

