%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

% Plots of Great Recession
close all
clc
clear all

RecessID = 2007;
%RecessID = 1979;

if RecessID == 2007
    fnameprefix = 'GreatRecess';
    startdate = 2007.25;
    enddate = 2011.5;
    peakdate = 2007.75;
else
    fnameprefix = 'Early80s';
    startdate = 1979.25;
    enddate = 1983.5;
    peakdate = 1980;
end

addpath(genpath('share'));

IsPresentation = false;


if IsPresentation
    FontSize = 14;
    LineWidth = 2;
else
    LineWidth = 1;
    FontSize = 12;
end

simstates = nan;
load(['data/' fnameprefix '_betaHet_1_dox_1_doTails_1.mat'],'c', 'y','simstates','CYPcts')
c_base = c;
y_base = y;


load(['data/' fnameprefix '_betaHet_1_dox_0_doTails_1.mat'],'c', 'y')
c_nox = c;
y_nox = y;


load(['data/' fnameprefix '_betaHet_1_dox_0_doTails_0.mat'],'c', 'y')
c_noTails = c;
y_noTails = y;


load(['data/' fnameprefix '_betaHet_0_dox_0_doTails_0.mat'],'c', 'y')
c_1bnoTails = c;
y_1bnoTails = y;

load(['data/' fnameprefix  '_RepAgent.mat'], 'c', 'y');
c_RA = c;
y_RA = y;




load('data/consumption_data.mat','c','t')
c = exp(log(c) - hpfilter(log(c),1600));
c_data = c(t>=startdate & t <= enddate);
t = t(t>=startdate & t <= enddate);

%% plot Figure 4

fig = figure;
p=plot(t, bsxfun(@rdivide,[c_base' c_nox' c_noTails' c_1bnoTails' c_RA' c_data],[c_base(3) c_nox(3) c_noTails(3) c_1bnoTails(3) c_RA(3) c_data(3)])-1);

disp('Drop at 2009Q1')
drp2009 = [c_base(8) c_nox(8) c_noTails(8) c_1bnoTails(8) c_RA(8) 1]./[c_base(3) c_nox(3) c_noTails(3) c_1bnoTails(3) c_RA(3) 1] - 1
disp('diff')
diff(drp2009)

set(p(1),'color','k')
set(p(1),'LineWidth',2)

set(p(2),'Marker','.')
set(p(2),'color','k')
set(p(2),'LineWidth',LineWidth)
set(p(2),'MarkerSize',20)

set(p(3),'color',repmat(0.5,1,3))
set(p(3),'LineWidth',LineWidth)

set(p(4),'color',repmat(0.5,1,3))
set(p(4),'LineWidth',LineWidth)
set(p(4),'Marker','+')
set(p(4),'MarkerSize',5)

set(p(5),'Marker','.')
set(p(5),'color',repmat(0.5,1,3))
set(p(5),'LineWidth',LineWidth)
set(p(5),'MarkerSize',20)

set(p(6),'color','k')
set(p(6),'LineStyle','--')
set(p(6),'LineWidth',LineWidth)

lgnd = legend('Baseline', 'Constant risk', 'No tails', 'No tails, common beta', 'Complete Markets','Data')    ;
set(lgnd,'Position',[ 0.582392857142857   0.778785714285714   0.323214285714286   0.201190476190476]);
   
pcons = p;



hold on
yl = ylim;
p2 = plot(repmat(peakdate,1,2),yl);
set(p2,'color',repmat(0.4,1,3))
txt = text(RecessID,-0.038,'NBER peak');
annotation('arrow',[0.15 0.254],[0.17, 0.17])
set(txt,'color',repmat(0.4,1,3))
set(txt,'FontSize',FontSize)
ylim(yl);
hold off

set(get(fig,'CurrentAxes'),'XTick',[RecessID:RecessID+4]);
set(get(fig,'CurrentAxes'),'FontSize',FontSize);




%% plot figure 6 --  consumption dispersion
fig = figure;
p = plot(t,[squeeze(CYPcts(4,1,1:end-1))./squeeze(CYPcts(2,1,1:end-1)) squeeze(CYPcts(2,1,1:end-1))./squeeze(CYPcts(1,1,1:end-1)) ...
    squeeze(CYPcts(4,2,1:end-1))./squeeze(CYPcts(2,2,1:end-1)) squeeze(CYPcts(2,2,1:end-1))./squeeze(CYPcts(1,2,1:end-1))]);

set(p(1),'LineStyle', '-')
set(p(2),'LineStyle', '--')
set(p(3),'LineStyle', '-')
set(p(4),'LineStyle', '--')


set(p(1),'color', 'k')
set(p(2),'color', 'k')
set(p(3),'color', repmat(0.7,1,3))
set(p(4),'color', repmat(0.7,1,3))

set(p(1),'LineWidth', LineWidth)
set(p(2),'LineWidth', LineWidth)
set(p(3),'LineWidth', 2)
set(p(4),'LineWidth', 2)


    
ylim([2.5 4])
set(get(fig,'CurrentAxes'),'XTick',[2007:2011]);
set(get(fig,'CurrentAxes'),'FontSize',FontSize);

tmp = RecessID + 0.5;
text(tmp,3.3,'Cons. 90/50','FontSize',FontSize)
text(tmp,2.9,'Cons. 50/20','FontSize',FontSize)
text(tmp,3.65,'Disp. Inc. 90/50','FontSize',FontSize)
text(tmp,3.45,'Disp. Inc. 50/20','FontSize',FontSize)