%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function Qp = QEvolution( Q, us, ul, zetap, lambdasp, lambdalp, x , Params, SteadyState)
%State evolution for Q variables
%
% input: Q is a 6 x 1 vector of the Q variables.  The format is 
%             Q(1) =  Q_{1, employed}, 
%             Q(2) =  Q_{1, unemp. short-term}, 
%             Q(3) =  Q_{1, unemp. long-term}, 
%             Q(4) =  Q_{1-by, employed}, 
%             Q(5) =  Q_{1-by, unemp. short-term}, 
%             Q(6) =  Q_{1-by, unemp. long-term}, 




if ~exist('SteadyState','var')
    SteadyState = false;
end


MUp  = get_MU( x,zetap, lambdasp, us,ul, Params );

if numel(Q) == 6
    Qxi = [1 ; exp(-0.5*(1-Params.by)*Params.sigmae^2+ 0.5*(1-Params.by)^2*Params.sigmae^2)];
    Qxi = kron( Qxi, [1;1;1]);
else 
    Qxi = 1;
end

Tr = [1-zetap   zetap   0;
      lambdasp   0      1-lambdasp;
      lambdalp   0      1-lambdalp];

Prm = [1-us-ul;  us;  ul];
Prmp = Tr' * Prm;


% conditional probability of m | m'
% m = row, m' = col
CP = Tr .* (Prm * ones(1,3)) ./ (ones(3,1) * Prmp');
assert(all(abs(sum(CP)-1) < 1e-8))



MixProbs = zeros(3,3,3); %i = mixture component, j = m, k = m'
MixProbs(2,1,2) = Params.p2;
MixProbs(3,:,1) = Params.p3;
MixProbs(1,:,:) = 1 - sum(MixProbs(2:3,:,:),1);
assert(all(all(abs(sum(MixProbs)-1) < 1e-8)))




% Component expectations
EComps = zeros(3,1);
tmp = [Params.sigma1;Params.sigma2;Params.sigma3]; 
EComps(1:3) = exp(MUp' + 0.5 * tmp.^2);



P = reshape( MixProbs,3,9)';  %unfold MixProbs for sum over EComps
if numel(Q) == 6
    EComps(4:6) = exp((1-Params.by)*MUp' + 0.5 * ((1-Params.by)*tmp).^2);
    P = kron(eye(2),P);  % replicate everything for v = 1 and v = 1-by
end
Qetap = P * EComps;


P = blkdiag(CP(:,1)',CP(:,2)',CP(:,3)'); % take sums over m weighted by conditional probabilities
P = kron(eye(2),P);  % replicate everything for v = 1 and v = 1-by
Qtemp = [repmat(Q(1:3),3,1); repmat(Q(4:6),3,1)];
Qp = Params.omega * Qxi + (1-Params.omega)* P * (Qtemp .*  Qetap);

if SteadyState
    for t = 1:10000
        Qtemp = [repmat(Qp(1:3),3,1); repmat(Qp(4:6),3,1)];
        Qp = Params.omega * Qxi + (1-Params.omega)* P * (Qtemp .*  Qetap);
    end
end

end



