%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [AggState, Dist, CYPcts] = SimulateDynamics(polfunc,S0,Dist,AggEps,Params,burn,DoCY,DoTodaFarmer)
% Simulates dynamics of the economy with aggregate shocks
% [AggState, Dist] = SimulateDynamics(polfunc,S0,Dist,AggEps,Params,burn)
%
% Inputs:
% polfunc    structure     policy function structure
% S0         1 x nstates   initial aggregate state to start simulation
% Dist       structure     initial distribution to start simulation
% AggEsp     nz x T        Draw of aggregate shocks
% Params     structure
% burn       scalar        drop burn dates at the beginning
%
% Outputs:
% AggState   nstates x (T-burn)    aggregate state at the beginning of each time period
% Dist       structure       final distribution
% CYPcts     5 x 2 x T     For each t, matrix of percentiles. rows are percentiles
%                            (20, 50, 80, 90, 95). columns are consumption, disposable income, earnings, total income

ik = 4;


if ~exist('burn','var')
    burn = Params.burn;
end

if ~exist('DoCY','var')
    DoCY = false;
end

if ~exist('DoTodaFarmer','var')
    DoTodaFarmer = false;
end

T = size(AggEps,2);



AggState = zeros(length(S0),T);
% if RiskySteadyState
%     AggEps = zeros(Params.nz,T);
% else
%     AggEps = diag(Params.Sigma(1:Params.nz))*randn(Params.nz,T);
% end


K = S0(ik);

S = S0;


if DoCY
    CYPcts = zeros(5,2,T);
end

ntheta = numel(Params.Sim.Grid1D_theta);
nk = numel(Params.Sim.Grid1D_k);



for t = 1:T
    
    if mod(t,50) == 0
        disp(['simulating t = ' num2str(t)])
    end
    
    
    
    %--------
    %these are the states at the beginning of period t
    Slag = S;
    S = StateUpdate(K, S, AggEps(:,t),Params);
    
    
    if DoCY
        [K2, Dist, CYPcts(:,:,t)] = SimulateCoreNonStoch(polfunc,S,Slag,Dist,Params,DoCY, DoTodaFarmer);
    else
        [K2, Dist] = SimulateCoreNonStoch(polfunc,S,Slag,Dist,Params,DoCY,DoTodaFarmer);
    end
    
    
    
    K = (1-Params.omega)*sum(Dist' * (exp(Params.Sim.theta*(1-Params.by)) .* Params.Sim.k));
    
    
    
    
    AggState(:,t) = S;
    
    
    
    
end

AggState = AggState(:,burn+1:end);

end
