%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function AggState = SimulateDynamicsRA(polfunc,S0,AggEps,Params,burn)
% Simulates dynamics of the economy with aggregate shocks
% [AggState, Dist] = SimulateDynamicsRA(polfunc,S0,AggEps,Params,burn)
%
% Inputs:
% polfunc    structure     policy function structure
% S0         1 x nstates   initial aggregate state to start simulation
% Dist       structure     initial distribution to start simulation
% AggEsp     nz x T        Draw of aggregate shocks
% Params     structure
% burn       scalar        drop burn dates at the beginning
%
% Outputs:  
% AggState   nstates x (T-burn)    aggregate state at the beginning of each time period
% Dist       structure       final distribution

ik = 4;
iQ = 7:12;


if ~exist('burn','var')
    burn = Params.burn;
end


T = size(AggEps,2);



AggState = zeros(length(S0),T);
% if RiskySteadyState
%     AggEps = zeros(Params.nz,T);
% else
%     AggEps = diag(Params.Sigma(1:Params.nz))*randn(Params.nz,T);
% end


K = S0(ik);

S = S0;


for t = 1:T

    if mod(t,500) == 0
        disp(['simulating t = ' num2str(t)])
    end
      
    %these are the states at the beginning of period t
    S = StateUpdate(K, S, AggEps(:,t),Params);
    AggState(:,t) = S;

    K = AMpolyint_step3(polfunc,S);
    
end

AggState = AggState(:,burn+1:end);

end
