%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [polfunc, Sgrid, PCGrid, simstates] = SolveRepAgent(polfunc, order, seed)
% main loop to solve the model for the rep agent economy

global Params;

maxit = [30 5000];

for it = 1:maxit(1)
    disp(['iteration ' num2str(it)])
    relax = 0.001;
    % if it >=4
    %     relax = 0.001;
    % end
    
    
    %fname = ['grid_' num2str(it) '.mat'];
    %Sgrid = Params.Sgrid;
    %PCGrid = Params.PCGrid;
    %eps = Params.eps;
    %save(fname,'Sgrid','PCGrid','eps');
    
    
    for i = 1:maxit(2)
        
        polfunc2 = updatepolfuncRepAgent(polfunc);
        tmp = AMpolyint_step3(polfunc2,Params.Sgrid) - AMpolyint_step3(polfunc,Params.Sgrid);
        tst = max(abs(tmp(:)));
        if mod(i,100) == 0
            tst
        end
        if any(isnan(polfunc2.coef))
            error('NaN encountered')
        end
        polfunc.coef = relax * polfunc2.coef + (1-relax) * polfunc.coef;
        if tst < 1e-6, break; end
        if tst < 1e-1, relax = 0.01; end
        
    end
    
    
    %Simulate
    rng(seed);
    AggEps = diag(Params.Sigma(1:Params.nz))*randn(Params.Tsim+Params.burn,Params.nz)';
    S0 = Params.Sgrid(1,:);
    simstates = SimulateDynamicsRA(polfunc,S0,AggEps,Params);
    clear AggEps;
    
    %update the grid
    [newGrid, neweps,newPCGrid] = EDS_get_grid(simstates,0.2);
    %check convergence
    converged = true;
    for i=1:size(newGrid,1)
        mindist = min(EDS_dist(Params.PCGrid,newPCGrid(i,:)));
        if mindist > (neweps+Params.eps);
            [mindist  (neweps+Params.eps)]
            converged = false;
            break
        end
    end
    if any(abs(mean(newGrid) - mean(Params.Sgrid))./std(Params.Sgrid) > 0.2)
        disp('means not converged')
        converged = false;
    end
    
    if converged
        disp('Grid has converged.')
        break
    else
        disp('updating grid')
        disp(['neweps = ' num2str(neweps)])
        Params.Sgrid = newGrid;
        Params.eps = neweps;
        Params.PCGrid = newPCGrid;
        polfunc = AMpolyint_step1( Params.Sgrid, order,polfunc.cls, polfunc );
    end
    
    
    
end

PCGrid = Params.PCGrid;
Sgrid = Params.Sgrid;




end

