%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function Sp = StateUpdate(forecastRule, S, eps,Params)


iz = 1;
izetahat = 2;
ilambdahat = 3;
iushat = 5;
iulhat = 6;

iQ = 7:12;
ix = 13;


z = S(iz);
zetahat = S(izetahat);
lambdahat = S(ilambdahat);

us = logistic(S(iushat));
ul = logistic(S(iulhat));
u = us + ul;


zp = Params.rhoz*z+eps(1);
zetahatp = Params.labor.m(1) + Params.labor.A(1,1) * (zetahat - Params.labor.m(1))  + Params.labor.B(1,1) * eps(2);
lambdahatp = Params.labor.m(2) + Params.labor.A(2,2) * (lambdahat - Params.labor.m(2))  + Params.labor.B(2,1) * eps(2) + Params.labor.B(2,2) * eps(3);

zetap = logistic(zetahatp);
lambdap = logistic(lambdahatp);

ushatp = invlogistic(   zetap*(1-u)  );
ulhatp = invlogistic(  (1-lambdap)*us + (1-lambdap*Params.labor.LambdaEllFactor)*ul   );


if  Params.dox 
    x = S(ix);
    xp = Params.labor.m(3) + Params.labor.A(3,3) * (x - Params.labor.m(3)) + Params.labor.B(3,:) * eps(2:4);
else
    x = 0;
    xp = [];
end

if Params.doTails
    Q = S(iQ);
    Qp = QEvolution( Q', us, ul, zetap, lambdap, Params.labor.LambdaEllFactor*lambdap, x, Params )';
else
    Qp = [];
end
    



if isstruct(forecastRule)
    Kp = AMpolyint_step3(forecastRule,S);
else
    Kp = forecastRule;
end


Sp = [zp zetahatp lambdahatp Kp ushatp ulhatp Qp xp];



end


