# -*- coding: utf-8 -*-
"""
MakeTables.py
Run this to create the welfare tables.

Created on Thu Mar 19 13:00:51 2015

@author: amckay
"""

import scipy.io as sio
import jinja2
import numpy as np

PRECISION = 3

formatstring = '{x:.' + str(PRECISION) + 'f}'

def FormatNumber(x):
    if type(x) == np.ndarray:
        return np.array([FormatNumber(xi) for xi in x])
    else:
        return formatstring.format(x = x)




env = jinja2.Environment(
        "%<", ">%",
        "<<", ">>",
        "[§", "§]",
        loader=jinja2.FileSystemLoader(".")
        )

template = env.get_template("moments_template.tex")

fname = '../data/moments_RepAgent.mat'
m = sio.matlab.loadmat(fname)
stdRA =  m['std_ycu'][0]

def WriteRow(betaHet,dox,doTails,rownum, rowname, f):
    if betaHet is None:
        fname = '../data/moments_RepAgent.mat'
    else:
        fname = '../data/moments_betaHet_' + str(betaHet) + '_dox_' + str(dox) + '_doTails_' + str(doTails) + '.mat'

    m = sio.matlab.loadmat(fname)

    f.write(template.render(rownum = rownum,
                            rowname = rowname,
                          std = FormatNumber(100* m['std_ycu'][0]),
                          corr = FormatNumber(m['cov_ycu']),
                          RelStd = FormatNumber(m['std_ycu'][0]/stdRA)
                          )
                )


fname = 'MomentTable.tex'
f =  open(fname, "w")

WriteRow(1,1,1,'(i)','Baseline',f)
WriteRow(1,0,1,'(ii)','Constant risk',f)
WriteRow(1,0,0,'(iii)','No tails',f)
WriteRow(0,0,0,'(iv)','Common-$\\beta$, no tails',f)  #note: python escape for \
WriteRow(None,None,None,'(v)','Complete markets',f)

f.close()
