%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

close all; clc;
clear all;

global Params;

Params.RepAgent = false;
Params.betaHet = true;
Params.dox = true;
Params.doTails = true;
initparams;

[K, us, ul, zeta, lambdas, Q] = CompleteMarketsSteadyState( Params );

S0 = [0 invlogistic(zeta) invlogistic(lambdas)  K  invlogistic(us)  invlogistic(ul)  Q  0 ];
K = S0(4);
S = S0;

%% Simulate unemployment durations

n = 4;  % 1+ max duration to track

% steady state
P = UnempDurationTransMat(Params.zetabar,Params.lambdabar,n);
U = (P')^1000;
U = U(:,1);


load('data/shocks_2007.mat','eps_orth');
AggEps = [zeros(1,size(eps_orth,2)); eps_orth];


T = size(AggEps,2);
D = zeros(n,T);

Z = load('~/Desktop/temp.mat');

for t = 1:T
    S = StateUpdate(K, S, AggEps(:,t),Params);
    %P = UnempDurationTransMat(logistic(S(2)),logistic(S(3)),n);
    P = UnempDurationTransMat(Z.zeta(t),Z.lambda(t),n);
    U = P' * U;
    D(:,t) = U;
end
%%
% sum 1 and 2 quarter duration
TT = 2007.25:0.25:2011.75;
U = zeros(2,T);
U(2,:) = sum(D(2:3,:));
U(1,:) = D(4,:);

p = area(TT,U');

hold on
p2=plot(TT,sum(U));
xlim([TT(1) TT(end)])
hold off


set(p(1),'LineStyle','None')
set(p(2),'LineStyle','None')
set(p(1),'FaceColor',repmat(0.5,1,3))
set(p(2),'FaceColor',repmat(0.8,1,3))


set(p2,'LineWidth',3)
set(p2,'color','k')


text(2009,0.01,'Long-term unemployed (6 months or more)')
text(2009,0.05,'Short-term unemployed (less than 6 months)')



%% Data plot

U = [0.007598015	0.037354904
0.008277133	0.038293443
0.008669316	0.039324331
0.008751711	0.041081053
0.009751614	0.043465943
0.011984558	0.048119393
0.015315509	0.053405002
0.019352706	0.063567094
0.026023122	0.066782047
0.033391262	0.062632394
0.038355112	0.060758768
0.041187773	0.057181024
0.043391178	0.053180744
0.040991701	0.053732307
0.041139117	0.054101558
0.039516976	0.050881477
0.039857245	0.050883206
0.040216489	0.049833145
0.037053869	0.049486673]';

figure
p = area(TT,U');

hold on
p2=plot(TT,sum(U));
xlim([TT(1) TT(end)])
hold off


set(p(1),'LineStyle','None')
set(p(2),'LineStyle','None')
set(p(1),'FaceColor',repmat(0.5,1,3))
set(p(2),'FaceColor',repmat(0.8,1,3))


set(p2,'LineWidth',3)
set(p2,'color','k')


text(2009,0.01,'Long-term unemployed (6 months or more)')
text(2009,0.05,'Short-term unemployed (less than 6 months)')



