%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function mpc = get_mpc(S0,D0, polfunc)

global Params;

[~, R, IPES] = static(S0, Params);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% form agrid at S
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

agridUE = AMpolyint_step3(polfunc,S0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% interpolate using MF codes on fine grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%MPC
if ~exist('fundefn','file')
    savedpath = path;
    addpath(genpath('~/MyLib/compecon'))
end
fspace = fundefn('spli',100000,Params.xmin,Params.xmax);
x = funnode(fspace);


g = cell(1,Params.npp);
linmpc = zeros(1,Params.npp);

for ip = 1:Params.npp

    
    ind = (ip-1)*Params.nkp;
    agrid = agridUE(ind+(1:Params.nkp));
    
    g{ip}=funfitxy(fspace,x,interpagridkpgrid(agrid,Params.kpgrid,x));

    % Find MPCs for linear part of the savings function    
    slp = Params.kpgrid(end)-interp1(agrid',Params.kpgrid',agrid(end)-1,'linear');
    linmpc(ip) = 1 - slp;
end


    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use budget constraint to get MPC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


k = Params.Sim.k;
%trans = exp(-Params.sigmae^2/2 + Params.sigmae*randn(Params.nsim,1));

mpc = zeros(Params.Sim.n,Params.npp);

for ip = 1:Params.npp
    I =  k <= Params.xmax;
    coh = get_coh(k(I),R,IPES,ip,ones(size(I)),Params);
    mpc(I,ip) = 1 - funeval(g{ip},fspace,coh,1);  % funeval(..) gives margingal propensity to save so 1-funeval is MPC
    mpc(~I,ip) = linmpc(ip);
    
end

mpc = dot(D0(:),mpc(:))/sum(D0(:));



%clean up
if exist('savedpath','var')
    path(savedpath)
    clear savedpath;
end

end