%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

disp('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
disp('      initializing parameters     ')
disp('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')



%These are short term and long-term unemp averages

% from VAR.m
Params.lambdabar = 0.7874;
Params.zetabar = 0.0453;



Params.alpha = 0.36;
Params.delta = 0.02;

Params.omega = 0.005;


%income process
Params.sigma1 = 0.063777918079094;
Params.sigma2 = 0.333728020205876;
Params.sigma3 = 0.333728020205876;
Params.sigmae = 0.154540058789747;


Params.p2 = 0.714516380391143;
Params.p3 = 0.018911617365324;

Params.mu10 = 0;
Params.mu11 = 0;
Params.mu20 = -0.166868386289571; 
Params.mu21 = 1;
Params.mu30 = 0.393951183726230;
Params.mu31 = 1;



Params.CRRA = 2;
Params.bus = 0.58;
Params.bul = 0.33;
Params.by = 0.151;

if ~Params.doTails
    Params.p2 = 0;
    Params.p3 = 0;
end

if Params.RepAgent
    Params.beta = 0.987906169947887;
else
    if Params.betaHet 
        if Params.doTails
            Params.beta = kron([0.967   0.99],ones(1,3));
        else
            Params.beta = kron([0.981   0.9923],ones(1,3));
        end
        Params.betaTrans = eye(2);
        Params.npp = 6; 
        %Params.laborstatus = kron(ones(2,1), [1:3]');  % 1 = employed, 2 = short-term unemmployed, 3 = long-term unemployed
        Params.employed = logical([1 0 0 1 0 0])';
        Params.employed_n = [1 Params.bus Params.bul 1 Params.bus Params.bul]';
        Params.fracLowBeta = 0.8;
    else
        if Params.doTails
            Params.beta = 0.97654;
        else
            Params.beta = 0.98552;
        end
        Params.betaTrans = 1;
        Params.npp = 3; 
        %Params.laborstatus = [1:3]';  % 1 = employed, 2 = short-term unemmployed, 3 = long-term unemployed
        Params.employed = logical([1 0 0])';
        Params.employed_n = [1 Params.bus Params.bul]';
        Params.fracLowBeta = 1;
    end
        
end





Params.rhoz = 0.962;
Params.Sigmaz = 0.0081;

Params.labor.A =  [     0.936278338228550                   0                   0
                          0   0.923984294759248                   0
                          0                   0   0.9222];
                
                
Params.labor.m =  [  -3.0485    1.3094    0.0000]';
                    
Params.labor.B = [      0.0428         0         0
                       -0.0380    0.0778         0
                        0.0331   -0.0017    0.0345];



if ~Params.dox
    Params.labor.Borig = Params.labor.B;
    Params.labor.B(3,:) = 0;
end
                    
Params.labor.Sigma = ones(3,1);                    

Params.labor.LambdaEllFactor = 0.5832;


Params.Sigma = [  Params.Sigmaz; Params.labor.Sigma];



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% QUADRATURE OVER INCOME SHOCKS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Params.neps = 5;
[Params.epsgrid1,Params.epsprob1] = qnwnorm(Params.neps,0,Params.sigma1^2);
[Params.epsgrid2,Params.epsprob2] = qnwnorm(Params.neps,0,Params.sigma2^2);
[Params.epsgrid3,Params.epsprob3] = qnwnorm(Params.neps,0,Params.sigma3^2);

Params.ntrans = 3;
[Params.transgrid,Params.transprob] = qnwnorm(Params.ntrans,-0.5*Params.sigmae^2,Params.sigmae^2);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Aggregate states
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%                     1     2           3          4     5         6      7     8       9      10      11    12     13
Params.aggnames =   {'z', 'zetahat', 'lambdahat', 'k', 'ushat', 'ulhat', 'Q11', 'Q12', 'Q13', 'Qb1', 'Qb2', 'Qb3', 'x'};
Params.States = true(1,13);
Params.nz = 4;
if Params.RepAgent || Params.by == 0
     % number of exogenous var
    Params.States(10:12) = false;
end
if ~Params.dox
    Params.States(13) = false;
    Params.nz = 3;
end
if ~Params.dox && ~Params.doTails
    Params.States(7:12) = false;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add some machinery to Params for the individual dimensions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Params.nc = 100; %number of points in approximation of individual cons func
Params.xmin = 0.001;
Params.xmax = 2000; % beyond that point, linear extrapolation is fine!
Params.nkp = 150;
Params.kpgrid = exp(linspace(0,log(Params.xmax),Params.nkp))-1;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Params.Tsim = 2000;
Params.burn = 200;

Params.Sim.Grid1D_k = Params.kpgrid';
Params.Sim.Grid1D_theta = [linspace(-3,5,40)'; (6:12)'];
[~, Params.Sim.k, Params.Sim.theta] = getprobs2D(1,1,1,Params.Sim.Grid1D_k',Params.Sim.Grid1D_theta');
Params.Sim.k = Params.Sim.k(:);
Params.Sim.theta = Params.Sim.theta(:);
Params.Sim.n = numel(Params.Sim.Grid1D_k) * numel(Params.Sim.Grid1D_theta);
assert(Params.Sim.n == size(Params.Sim.k,1))
assert(Params.Sim.n == size(Params.Sim.theta,1))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Storage etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if Params.RepAgent
    Params.fnameSuffix = '_RepAgent';
else
    Params.fnameSuffix = ['_betaHet_' num2str(Params.betaHet) '_dox_' num2str(Params.dox) '_doTails_' num2str(Params.doTails)];
end
