%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function probs = getprobs1D(x,p,X)
% Takes a set of points x with probabilities p and returns a new set of
% probabilities on the grid given by  X.
% All inputs should be row vectors.
%
% Ouputs (let m = length(X) )
% probs are the associated probabilities of the points in X ( 1 x m array)
%
% Only works with matlab version R2014b or later.


m = length(X); 
k = size(x,2);


% check the inputs
if any(diff(X)<=0) 
    error('X must be strictly increasing grid');
end

if min(x) < X(1) || max(x) > X(end) 
    error('x must lie in X ');
end

if size(x,1) > 1  || size(p,1) > 1 || size(X,1) > 1 
    error('All inputs must be scalar or row vector')
end

if  size(p,2) ~= k
    error('x and p must be the same size')
end

if any(p > 1) || any(p <0)
    error('Input p is not a valid probability distribution (an element is less than zero or greater than one).')
end


% these lines only work with Matlab R2014b or later
indx = interp1(X,1:m,x,'previous')+1; %use previous + 1 not next because of treatment when x is exactly equal to an element of X


% if you are using old Matlab and you have the CompEcon tool box then you
% can use these lines instead
%indx = lookup(X,x)+1;
%indy = lookup(Y,y)+1;

% the line above has an issue if the index is right on the last grid point
% so we correct it here
indx = min(indx,m);


assert(all(indx >1))



% allocate in the style of Young (2010)
% http://dx.doi.org/10.1016/j.jedc.2008.11.010
% Define
Xprev = X(indx - 1);
Xnext = X(indx);
% solve:
% (p-pi) * Xprev + pi*  Xnext = p*x
%  pi*( Xnext - Xprev) = p*(x-Xprev)
pnext = p.*(x -  Xprev) ./ (Xnext - Xprev);

% accumulate the results
probs = accumarray([indx-1 indx]',[p-pnext pnext]',[m 1])';


end
