%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function polfunc2 = updatepolfunc(polfunc,forecastRule,Params)
% non-linear solution--update the policy function using one endog. grid
% method step.
% polfunc is the policy function in t+1 and the output is the policy
% function in t
%




Sgrid = Params.Sgrid;
nkp = Params.nkp;
kpgrid = Params.kpgrid;



if Params.dox
    ix = nameInd('x',Params.aggnames);
else
    ix = [];
end
iushat = nameInd('ushat',Params.aggnames);
iulhat = nameInd('ulhat',Params.aggnames);

npp = Params.npp;
aofS = zeros(size(Sgrid,1),npp*nkp);



ComponentToDo = false(6,3);  % columns refer to components and rows to labor market status
ComponentToDo(1,1) = true;
ComponentToDo(1,3) = true;
ComponentToDo(2,2) = true;
ComponentToDo(2,1:2) = true;
ComponentToDo(3,1) = true;
ComponentToDo(4:6,:) = ComponentToDo(1:3,:);  %repeat everything for beta heterogeneity





var = diag(Params.Sigma)^2;
if ~Params.dox
    var = var(1:3,1:3);
end

%--
%Monomial rule
[~,quadEps,quadW] = Monomials_1(Params.nz,var);
%--


CRRA = Params.CRRA;
beta = Params.beta;
dox = Params.dox;

%parfor here
parfor si = 1:size(Sgrid,1)  % loop over current state
    
    % unpack
    Statei = Sgrid(si,:);
    if dox
        x = Statei(ix);
    else
        x = 0;
    end
    us = logistic(Statei(iushat));
    ul = logistic(Statei(iulhat));
    
    
    % initialize
    MUexp = zeros(nkp,npp);
    
    % loop over aggregate shocks
    for epsi = 1:size(quadEps,1)
        
        % future aggregate state
        Sp =  StateUpdate(forecastRule,Statei, quadEps(epsi,:)',Params);
        
        
        % future prices amd labor transition probabilities
        [~, Rp, IPESp, lambdap, zetap] = static(Sp, Params);
        
        % get future policy rule
        agridUE = AMpolyint_step3(polfunc,Sp);  % 1 x 900,  lists the coh that leads to kpgrid
        
        % shocks depend on labor transitions
        [ epsgrid, transgrid, probs, nquad  ] = ShockGrid( x,zetap, lambdap, us,ul,Params ,true);  %this gives shocks raised to the power (1-by)
        
        
        
        MUnext = zeros( nkp,npp,3);  %last dimension corresponds to income mixture components
        for jp = 1:npp %loop over future period income state
            
            
            ind = (jp-1)*nkp;
            agrid = agridUE(ind+ (1:nkp));
            
            income = IPESp * transgrid * Params.employed_n(jp);
            
            for component = find(ComponentToDo(jp,:))  
                coh = repmat(income,1,nkp) +  repmat(Rp*kpgrid,nquad,1)./repmat(epsgrid(:,component),1,nkp);  % this is a nquad x nkp matrix of coh/e^((1-by)theta')
                coh = coh(:);

                kp = interpagridkpgrid(agrid,kpgrid,coh);  % for each coh, find how much we save
                
                c = coh- kp;  % consumption
                
                assert(all(c >0))%
                
                %expected marginal utility times e^{-gamma (1-by) eps'}.  the
                %expectation is taken over idiosyncratic skill shocks
                %conditional on component
                MUnext(:,jp,component) = Rp.*sum(repmat(probs(:,component),1,nkp).*reshape((c.*repmat(epsgrid(:,component),nkp,1)).^(-CRRA),nquad,nkp))';
            end
            
        end
        
        
        
        %now we form expected marginal utility over employment shocks
        for ip=1:npp  %loop over previous income states
            for jp = 1:npp %loop over this period income state
                for component = 1:3 % loop over mixture component
                    pp = transProb(ip,jp,lambdap,lambdap*Params.labor.LambdaEllFactor,zetap,Params.betaTrans) ...
                          * ComponentProbs(ip,jp,component,Params.p2,Params.p3);   
                
                    if pp > 0
                        MUexp(:,ip) = MUexp(:,ip) + quadW(epsi)*pp*MUnext(:,jp,component);
                    end
                end
            end
        end
        
        
        
        
        
    end  %end loop over aggregate shocks
    
    
    MUprev = bsxfun(@times,MUexp,beta);
    %MUprev = MUexp * beta;
    
    %compute cash on hand associated with each level of savings
    % K + C = X (cash on hand)
    % k + c = x
    aofS(si,:) = MUprev(:).^(-1/CRRA) + repmat(kpgrid',npp,1);
    
    
end


polfunc2 = AMpolyint_step2(polfunc,aofS);


end