%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [polfunc2] = updatepolfuncRepAgent(polfunc)
% non-linear solution--update the policy function using one endog. grid
% method step.
%
%
global Params;

KpofS = zeros(length(Params.Sgrid),1);

%n = [2 2  2 2 ];
%n = [1 1 1 1];

%mu = zeros(1,4);
var = diag(Params.Sigma)^2;
%[quadEps, quadW] = qnwnorm(n,mu,var);

%--
%Monomial rule
[~,quadEps,quadW] = Monomials_1(Params.nz,var);
%--


indk = nameInd('k',Params.aggnames);
indz = nameInd('z',Params.aggnames);
indushat = nameInd('ushat',Params.aggnames);
indulhat = nameInd('ulhat',Params.aggnames);
indQ11 = nameInd('Q11',Params.aggnames);

parfor si = 1:length(Params.Sgrid)
    
    MU = 0;
    MUp = 0;
    
    z = Params.Sgrid(si,indz);
    k = Params.Sgrid(si,indk);
    us = logistic(Params.Sgrid(si,indushat));
    ul = logistic(Params.Sgrid(si,indulhat));
    u = us + ul;
    Q11 = Params.Sgrid(si,indQ11);
    
    MUexp = 0;
    
    kp = AMpolyint_step3(polfunc,Params.Sgrid(si,:));
    L = (1-u) * Q11; 
    y = exp(z).*k.^Params.alpha*L.^(1-Params.alpha);
    c = y + (1-Params.delta)*k - kp;
    
    if c > 1e-6
        MU = c.^(-Params.CRRA);
    else
        error('negative consumption')
    end
    
    for epsi = 1:size(quadEps,1)
        
        Sp = StateUpdate(kp, Params.Sgrid(si,:), [quadEps(epsi,:) 0]', Params);
        [~, Rp] = static(Sp, Params);
        up = logistic(Sp(indushat)) + logistic(Sp(indulhat));
        zp = Sp(indz);
        Q11p = Sp(indQ11);
        Lp = (1-up) * Q11p;  
        
        kpp = AMpolyint_step3(polfunc,Sp);
        yp = exp(zp).*kp.^Params.alpha*Lp.^(1-Params.alpha);
        cp = yp + (1-Params.delta)*kp - kpp;
        
        if cp > 1e-6
            MUp = cp.^(-Params.CRRA);
        else
            error('negative consumption')
        end
        
        
        
        MUexp = MUexp + quadW(epsi) *Rp*MUp;
        assert(all(abs(imag(MUexp)) < 1e-10))
    end
    
    MUexp = Params.beta*MUexp;
    
    
    %compute consumption associated with each level of savings
    KpofS(si) = (MUexp/MU * kp);
    
    
end




polfunc2 = AMpolyint_step2(polfunc,KpofS);

end